#!/bin/sh

temp_log_path="temp.log"
default_log_path="/var/log/sysak/syscall_slow/syscall_slow.log"
# sysak syscall_slow -h
#Usage: syscall_slow [OPTION...]
#Catch the delay of a syscall more than threshold.

#USAGE: syscall_slow [--help] [-t THRESH(ms)] [-n sys_NR] <[-c COMM] [-p tid]>
#[-f LOGFILE] [duration(s)]

#EXAMPLES:
#    syscall_slow            # run forever, detect delay more than
#10ms(default)
#    syscall_slow -t 15      # detect with threshold 15ms (default 10ms)
#    syscall_slow -p 1100    # detect tid 1100 (use for threads only)
#    syscall_slow -c bash    # trace aplication who's name is bash
#    syscall_slow -n 101,102 # Exclude syscall 101 and 102 from tracing.
#    syscall_slow -f a.log   # log to a.log (default to
#~sysak/syscall_slow.log)

#  -c, --comm=COMM            Name of the application
#  -f, --logfile=LOGFILE      logfile for result
#  -n, --sysnr=SYSNR          Exclude the syscall ID from tracing
#  -p, --pid=TID              Thread TID to detect
#  -t, --threshold=THRESH     Threshold to detect, default 10ms
#  -v, --verbose              Verbose debug output
#  -?, --help                 Give this help list
#      --usage                Give a short usage message
#  -V, --version              Print program version

handle_error() {
    echo "Test Failed: $1"
    reset
    exit 1
}

reset() {
    pkill syscall_slow
    rm -rf $temp_log_path
}

# syscall_slow --help
test_help() {
    reset
    val=$(sysak -g syscall_slow --help | grep 'USAGE: syscall_slow' -c)

    if test $val -eq 1; then
        echo "Test help success."
    else
        handle_error "sysak -g syscall_slow --help"
    fi
}

test_file() {
    reset
    sysak -g syscall_slow -f $temp_log_path &
    sleep 5
    pkill syscall_slow --signal SIGINT
    num=$(cat $temp_log_path | grep "TIME(syscall)" -c)
    if test $num -ne 0; then
        echo "Test file success."
    else
        handle_error "sysak -g syscall_slow -f"
    fi
}

test_normal() {
    reset
    sysak -g syscall_slow &
    sleep 5
    pkill syscall_slow --signal SIGINT
    num=$(cat $default_log_path | grep "TIME(syscall)" -c)
    if test $num -ne 0; then
        echo "Test normal success."
    else
        handle_error "sysak -g syscall_slow"
    fi
}

test_comm() {
    reset
    sysak -g syscall_slow -c sleep &
    sleep 2 &
    sleep 2 &
    sleep 4

    pkill syscall_slow --signal SIGINT
    num=$(cat $default_log_path | grep -v "<0x" | grep -v "TIME(syscall)" | grep sleep | wc -l)
    if test $num -ge 3; then
        echo "Test comm success."
    else
        handle_error "sysak -g syscall_slow"
    fi
}

test_pid() {
	#todo
	sleep 6 &
	pid=$(pidof sleep|awk '{print $1}')
	sysak -g syscall_slow -p $pid -f $default_log_path >/dev/null 2>&1 &
	sleep 1
	pkill syscall_slow --signal SIGINT
	num=$(cat $default_log_path | grep "TIME(syscall)" -c)
	if test $num -ne 0; then
		echo "Test pid success."
	else
		handle_error "sysak -g syscall_slow -p"
	fi
#	num=$(cat $default_log_path |grep -v "<0x" |grep -v "TIME(syscall)"|awk '{print $10}' |grep -v $pid| wc -l)
#	if test $num -eq 0; then
#		echo "Test pid success."
#	else
#		handle_error "sysak -g syscall_slow -p"
#	fi
}

test_nr() {
	if [ ! -f "/usr/include/asm/unistd_64.h" ]; then
		handle_error "sysak -g syscall_slow -n (1)"
	fi
	nr1=$(cat "/usr/include/asm/unistd_64.h" | grep "__NR_nanosleep" | awk '{print $3}')
	nr2=$(cat "/usr/include/asm/unistd_64.h" | grep "__NR_clock_nanosleep" | awk '{print $3}')
	sysak -g syscall_slow -n $nr1,$nr2 -f $default_log_path >/dev/null 2>&1 &
	sleep 1
	pkill syscall_slow --signal SIGINT
	
	num=$(cat $default_log_path |grep -v "<0x" |grep -v "TIME(syscall)"|awk '{print $9}' |grep -w nanosleep| wc -l)
	if test $num -gt 0; then
		handle_error "sysak -g syscall_slow -n (2)"
	else
		echo "Test nr success."
	fi
}

test_verbose() {
    reset
    sysak -g syscall_slow -v -f $temp_log_path >/dev/null 2>&1 &
    sleep 5
    pkill syscall_slow --signal SIGINT

    num=$(cat $temp_log_path | grep "TIME(syscall)" -c)
    if test $num -ne 0; then
        echo "Test verbose success."
    else
        handle_error "sysak -g syscall_slow -v -f $temp_log_path"
    fi
}

test_threshold() {
    reset
    sysak -g syscall_slow -t 1000 -f $temp_log_path &
    sleep 5
    pkill syscall_slow --signal SIGINT
    num=$(cat $temp_log_path | grep "TIME(syscall)" -c)
    if test $num -ne 0; then
        echo "Test threshold success."
    else
        handle_error "sysak -g syscall_slow -t 1000 -f $temp_log_path"
    fi
}

main() {
    test_help
    test_file
    test_normal
    test_comm
    test_pid
    test_nr
    test_verbose
    test_threshold
    reset
}

main
