#!/bin/sh
#
# usage: iofsstat.py [-h] [-T TIMEOUT] [-t TOP] [-u UTIL_THRESH] [-b BW_THRESH]
#                    [-i IOPS_THRESH] [-c CYCLE] [-d DEVICE] [-p PID] [-j JSON]
#                    [-f] [-P] [-n] [-m]

# Report IO statistic for partitions.

# optional arguments:
#   -h, --help            show this help message and exit
#   -T TIMEOUT, --Timeout TIMEOUT
#                         Specify the timeout for program exit(secs).
#   -t TOP, --top TOP     Report the TopN with the largest IO resources.
#   -u UTIL_THRESH, --util_thresh UTIL_THRESH
#                         Specify the util-thresh to report.
#   -b BW_THRESH, --bw_thresh BW_THRESH
#                         Specify the BW-thresh to report.
#   -i IOPS_THRESH, --iops_thresh IOPS_THRESH
#                         Specify the IOPS-thresh to report.
#   -c CYCLE, --cycle CYCLE
#                         Specify refresh cycle(secs).
#   -d DEVICE, --device DEVICE
#                         Specify the disk name.
#   -p PID, --pid PID     Specify the process id.
#   -j JSON, --json JSON  Specify the json-format output.
#   -f, --fs              Report filesystem statistic for partitions.
#   -P, --Pattern         Report IO pattern(--fs not support).
#   -n, --nodiskStat      Not report disk stat.
#   -m, --misc            Promiscuous mode.

# e.g.
#   ./iofsstat.py -d vda -c 1
#             Report iops and bps of process for vda per 1secs
#   ./iofsstat.py -d vda1 --fs -c 1
#             Report fs IO-BW and file of process for vda1(must be parttion mounted by filesystem) per 1secs
#   ./iofsstat.py -m -c 5 -t 5
#             Report top5 iops&&bps&&file of process with misc mode per 5secs
#   ./iofsstat.py -d vda -c 1 -b 1048576 -i 350
#             Report process that iops over 350 or bps over 1048576 for vda per 1secs
#   ./iofsstat.py -u 90
#             Report disk that io-util over %90

temp_log_path="iofsstat_tmp.log"

function clean() {
    rm $temp_log_path
}

function handle_failed() {
    echo "Test iofsstat failed." 
    cat $temp_log_path
    clean
    exit 1
}

function test_help() {
    sysak iofsstat -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "usage: iofsstat" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test iofsstat helpinfo success."
    fi
}

function test_iofsstat_iops() {
    sysak iofsstat > $temp_log_path 2>&1 && cat $temp_log_path | grep "iops_rd" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak iofsstat success."
    fi
}

function test_iofsstat_fs() {
    sysak iofsstat -f > $temp_log_path 2>&1 && cat $temp_log_path | grep "filepath" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak iofsstat -f success."
    fi
}

function test_iofsstat_misc() {
    sysak iofsstat -m > $temp_log_path 2>&1 && cat $temp_log_path | grep "device-stat" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak iofsstat -m success."
    fi
}


function main() {
    test_help
    test_iofsstat_iops
    test_iofsstat_fs
    test_iofsstat_misc
    clean
}

main