#!/bin/sh

# USAGE: offcputime [--help] [-p PID | -u | -k] [-m MIN-BLOCK-TIME] [-M
# MAX-BLOCK-TIME] [--state] [--perf-max-stack-depth] [--stack-storage-size]
# [duration]
# EXAMPLES:
#     offcputime             # trace off-CPU stack time until Ctrl-C
#     offcputime 5           # trace for 5 seconds only
#     offcputime -f 5        # trace for 5 seconds and foled the output
#     offcputime -m 1000     # trace only events that last more than 1000 usec
#     offcputime -M 10000    # trace only events that last less than 10000 usec
#     offcputime -p 185      # only trace threads for PID 185
#     offcputime -t 188      # only trace thread 188
#     offcputime -u          # only trace user threads (no kernel)
#     offcputime -k          # only trace kernel threads (no user)

#   -f, --folded-output        Folded output
#   -k, --kernel-threads-only  Kernel threads only (no user threads)
#   -m, --min-block-time=MIN-BLOCK-TIME
#                              the amount of time in microseconds over which we
#                              store traces (default 1)
#   -M, --max-block-time=MAX-BLOCK-TIME
#                              the amount of time in microseconds under which we
#                              store traces (default U64_MAX)
#   -p, --pid=PID              Trace this PID only
#       --perf-max-stack-depth=PERF-MAX-STACK-DEPTH
#                              the limit for both kernel and user stack traces
#                              (default 127)
#       --stack-storage-size=STACK-STORAGE-SIZE
#                              the number of unique stack traces that can be
#                              stored and displayed (default 1024)
#       --state=STATE          filter on this thread state bitmask (eg, 2 ==
#                              TASK_UNINTERRUPTIBLE) see include/linux/sched.h
#   -t, --tid=TID              Trace this TID only
#   -u, --user-threads-only    User threads only (no kernel threads)
#   -v, --verbose              Verbose debug output
#   -?, --help                 Give this help list
#       --usage                Give a short usage message
#   -V, --version              Print program version


temp_log_path="offcputime.log"

function clean() {
    pkill offcputime
    rm $temp_log_path
}

function check_fail() {
    check_start=`cat $temp_log_path | grep "failed to load kallsyms" | wc -l`
    check_syms_cache=`cat $temp_log_path | grep "failed to create syms_cache" | wc -l`
    check_bpf=`cat $temp_log_path | grep "failed to attach BPF programs" | wc -l`
    if (test $check_start -ne 0) || (test $check_syms_cache -ne 0) || (test $check_bpf -ne 0)
    then
        handle_failed  
    fi    
}

function handle_failed() {
    echo "Test offcputime failed." 
    clean
    exit 1
}

function test_help() {
    sysak -g offcputime --help > $temp_log_path 2>&1 && cat $temp_log_path | grep "USAGE: offcputime" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test offcputime --help success."
    fi
}

function test_all() {
    sysak -g offcputime > $temp_log_path 2>&1 &
    sleep 5
    kill -INT `pidof offcputime`
    check_fail
    echo "Test offcputime success."
}

function test_time() {
    sysak -g offcputime 1 > $temp_log_path 2>&1 &
    sleep 5

    pid1=`pidof offcputime`
    if [ $pid1 ]; then  
        echo "sysak -g offcputime 1 failed"  
        handle_failed
    fi 

    check_fail
    echo "Test offcputime 1 success."
}


function test_file() {
    sysak -g offcputime -f > $temp_log_path 2>&1 &
    sleep 5
    kill -INT `pidof offcputime`
    check_fail
    echo "Test offcputime -f success."
}

function test_pid() {
    pid1=`pidof mysqld`
    if [ ! $pid1 ]; then  
        echo "No mysqld diagnose pid=1"  
        pid1=1
    else  
        echo "Diagnose mysqld"  
    fi 

    sysak -g offcputime -p $pid1 > $temp_log_path 2>&1 &
    sleep 5
    kill -INT `pidof offcputime`
    check_fail
    echo "Test offcputime -p success."
}

function test_thread() {
    pid1=`pidof mysqld`
    if [ ! $pid1 ]; then  
        echo "No mysqld diagnose pid=1"  
        pid1=1
    else  
        echo "Diagnose mysqld"  
    fi 

    sysak -g offcputime -t $pid1 > $temp_log_path 2>&1 &
    sleep 5
    kill -INT `pidof offcputime`
    check_fail
    echo "Test offcputime -t success."
}

function test_user() {
    sysak -g offcputime -u > $temp_log_path 2>&1 &
    sleep 5
    kill -INT `pidof offcputime`
    check_fail
    echo "Test offcputime -u success."
}

function test_kernel() {
    sysak -g offcputime -k > $temp_log_path 2>&1 &
    sleep 5
    kill -INT `pidof offcputime`
    check_fail
    echo "Test offcputime -k success."
}

function test_m() {
    sysak -g offcputime -m 1000 > $temp_log_path 2>&1 &
    sleep 5
    kill -INT `pidof offcputime`
    check_fail
    echo "Test offcputime -m success."
}

function test_M() {
    sysak -g offcputime -M 10000 > $temp_log_path 2>&1 &
    sleep 5
    kill -INT `pidof offcputime`
    check_fail
    echo "Test offcputime -M success."
}


function main() {
    test_help
    test_all
    test_time
    test_file
    test_pid
    test_thread
    test_user
    test_kernel
    test_m 
    test_M
    clean
}

main