#!/bin/sh

default_log_path="/var/log/sysak/tasktop/tasktop.log"
temp_log_path="temp.log"
redirect_log_path="redirect.log"

# USAGE: tasktop [--help] [-t] [-p TID] [-d DELAY] [-i ITERATION] [-s SORT] [-f
# LOGFILE] [-l LIMIT] [-H] [-e D-LIMIT]

# EXAMPLES:
#     tasktop            # run forever, display the cpu utilization.
#     tasktop -t         # display all thread.
#     tasktop -p 1100    # only display task with pid 1100.
#     tasktop -d 5       # modify the sample interval.
#     tasktop -i 3       # output 3 times then exit.
#     tasktop -s user    # top tasks sorted by user time.
#     tasktop -l 20      # limit the records number no more than 20.
#     tasktop -e 10      # limit the d-stack no more than 10, default is 20.
#     tasktop -H         # output time string, not timestamp.
#     tasktop -f a.log   # log to a.log.
#     tasktop -e 10      # most record 10 d-task stack.

#   -d, --delay=DELAY          Sample peroid, default is 3 seconds
#   -e, --d-limit=D-LIMIT      Specify the D-LIMIT D tasks's stack to display
#   -f, --logfile=LOGFILE      Logfile for result, default
#                              /var/log/sysak/tasktop/tasktop.log
#   -H, --human                Output human-readable time info.
#   -i, --iter=ITERATION       Output times, default run forever
#   -l, --r-limit=LIMIT        Specify the top R-LIMIT tasks to display
#   -p, --pid=TID              Specify thread TID
#   -s, --sort=SORT            Sort the result, available options are user, sys
#                              and cpu, default is cpu
#   -t, --thread               Thread mode, default process
#   -?, --help                 Give this help list
#       --usage                Give a short usage message
#   -V, --version              Print program version

# Mandatory or optional arguments to long options are also mandatory or optional
# for any corresponding short options.

reset() {
    rm -rf $default_log_path
    rm -rf $redirect_log_path
    rm -rf $temp_log_path
}

test_sort() {
    reset
    sysak -g tasktop --sort user --iter 1 >/dev/nul 2>&1
    times=$(cat $default_log_path | grep TIME-STAMP | wc -l)
    if test $times -ne 1; then
        echo "Test sort failed."
        exit 1
    else
        reset
        echo "Test sort success."
    fi
}

test_dlimit() {
    reset
    sysak -g tasktop -e 5 --iter 1 >/dev/nul 2>&1
    times=$(cat $default_log_path | grep TIME-STAMP | wc -l)
    if test $times -ne 1; then
        echo "Test d-limit failed."
        exit 1
    else
        reset
        echo "Test d-limit success."
    fi
}

test_reclimit() {
    reset
    sysak -g tasktop -l 10 --iter 1 >/dev/nul 2>&1
    times=$(cat $default_log_path | grep TIME-STAMP | wc -l)
    if test $times -ne 1; then
        echo "Test rec-limit failed."
        exit 1
    else
        reset
        echo "Test rec-limit success."
    fi
}

test_thread() {
    reset
    sysak -g tasktop -t --iter 1 >/dev/nul 2>&1
    times=$(cat $default_log_path | grep TIME-STAMP | wc -l)
    if test $times -ne 1; then
        echo "Test thread failed."
        exit 1
    else
        echo "Test thread success."
    fi
}

test_human() {
    reset
    sysak -g tasktop -H --iter 1 >/dev/nul 2>&1
    times=$(cat $default_log_path | grep TIME-STAMP | wc -l)
    if test $times -ne 1; then
        echo "Test human failed."
        exit 1
    else
        reset
        echo "Test human success."
    fi
}

test_help() {
    reset
    sysak -g tasktop --help >$temp_log_path 2>&1 
    times=$(cat $temp_log_path | grep "Usage: tasktop" | wc -l)  

    if test $times -ne 1; then
        echo "Test help failed."
        exit 1
    else
        reset
        echo "Test help success."
    fi
}

test_no_options() {
    reset
    sysak -g tasktop l >/dev/nul 2>&1 &
    sleep 10
    pkill tasktop

    times=$(cat $default_log_path | grep TIME-STAMP | wc -l)
    # echo $times
    if test $times -eq 0; then
        echo "Test no_options failed"
        exit 1
    else
        echo "Test no_option success"
    fi
}

test_delay() {
    reset
    sysak -g tasktop --delay 1 l >/dev/nul 2>&1 &
    sleep 5
    pkill tasktop

    times=$(cat $default_log_path | grep TIME-STAMP | wc -l)
    # echo $times
    if test $times -gt 1; then
        echo "Test delay success"
    else
        echo "Test delay failed"
        exit 1
    fi
}

test_iter() {
    reset
    sysak -g tasktop --delay=1 --iter=3 l >/dev/nul 2>&1
    times=$(cat $default_log_path | grep TIME-STAMP | wc -l)
    if test $times -ne 3; then
        echo "Test iter failed"
        exit 1
    else
        echo "Test iter success"
    fi
}

test_logfile() {
    reset
    sysak -g tasktop --delay=1 --iter=2 --logfile=$redirect_log_path l >/dev/nul 2>&1
    times=$(cat $redirect_log_path | grep TIME-STAMP | wc -l)

    if test $times -ne 2; then
        echo "Test logfile failed"
        exit 1
    else
        echo "Test logfile success"
    fi
}

test_pid() {
    reset
    stress -c 1 >/dev/null 2>&1 &
    pid1=$!
    stress -c 1 >/dev/null 2>&1 &
    pid2=$!
    sysak -g tasktop --iter=1 -p $pid1 l >/dev/nul 2>&1
    times=$(cat $default_log_path | grep $pid2 | wc -l)
    pkill stress
    if test $times -ne 0; then
        echo "Test pid failed"
        exit 1
    else
        echo "Test pid success"
    fi
}

test_blocked_timeout() {
    reset

    sysak tasktop --mode blocked --threshold 1000 --kthread --run 10 >/dev/nul 2>&1

    times=$(cat $default_log_path | grep "Stack|Delya(ms)" | wc -l)

    if test $times -ne 1; then
        echo "Test blocked_timeout start failed"
        exit 1
    else
        echo "Test blocked_timeout start success"
    fi
}

main() {
    test_help
    test_no_options
    test_delay
    test_iter
    test_pid
    test_logfile
    test_human
    test_sort
    test_dlimit
    test_reclimit
    test_thread
    test_blocked_timeout
    reset
}

main
